/*
                        Copyright (c) 1994 by
                        Advanced Visual Systems Inc.
                        All Rights Reserved
        
        This software comprises unpublished confidential information of
        Advanced Visual Systems Inc. and may not be used, copied or made
        available to anyone, except in accordance with the license
        under which it is furnished.
        
        This file is under Perforce control
        $Id: //depot/express/fcs992/dbxx/elem_c.h#1 $
*/
#if !defined(dbxx_h) || !defined(elem_c_h)
#define dbxx_h         1
#define elem_c_h     1

#include <assert.h>
#include <avs/db.h>

#ifndef True
#    define True         1
#endif

#ifndef False
#    define False        0
#endif

#ifdef CRAY
typedef long            Boolean;
#else
typedef char            Boolean;
#endif



/////////////////////////////////////
class DBcallback {
/////////////////////////////////////
  public:
                        DBcallback (
                            DBpfi, 
                            void *clientData, 
                            DBtype_id type = 0, 
                            char *name = "");
                        ~DBcallback ();

                        operator DBelem_id () const;
  private:
      DBelem_id         _id;
};

inline
DBcallback::DBcallback (
    DBpfi func, void *clientData, DBtype_id type, char *name)
{
    _id = DBcreate_callback_elem (DBstr_to_name (name), func, type, clientData);
}

inline
DBcallback::~DBcallback ()
{
    if (!DBis_null_elem(_id)) (void) DBdestroy_elem(_id,0);
}

inline
DBcallback::operator DBelem_id () const
{
    return _id;
}

/////////////////////////////////////
class DBelem_c {
/////////////////////////////////////
  public:
                        DBelem_c (
                            DBelem_id id, 
                            char *name = 0,
                            int mode = DB_ELEM_RW);
                        DBelem_c (
                            char *objType, 
                            char *name = "", 
                            DBelem_id parent = DBinst_elem,
                            int mode = DB_ELEM_RW);
    virtual                ~DBelem_c ();

    DBelem_id           id () const;

    int                 destroy ();

    const char *        name () const;
    void                print (FILE *fp = 0) const;

    virtual             operator DBelem_id () const;
    Boolean             operator== (DBelem_id rhs) const;
    Boolean             operator!= (DBelem_id rhs) const;

    virtual Boolean     isNil () const;
    Boolean             isValid () const;
    Boolean             inSameProcess (DBelem_id) const;
    unsigned int        getSeq (
                            int flags = DB_SEQ_VAL, 
                            DBelem_id tmplt = DBnull_elem
                        );

    virtual Boolean     match (DBelem_id matchElem, int mode = DB_MATCH_POSS);
    virtual Boolean     match (char * tmplClass, int mode = DB_MATCH_POSS);

    void                pushCtx ();
    void                popCtx ();

    void                addNotify (
                        DBcallback *cb, 
                        DBelem_id context = DBnull_elem,
                        unsigned int mask = DB_EVENT_VAL,
                        DBelem_id tplt = DBnull_elem,
                        int mode = 0);
    void                delNotify (
                            DBcallback *cb,
                            DBelem_id context = DBnull_elem,
                            unsigned int mask = DB_EVENT_VAL,
                            DBelem_id tplt = DBnull_elem,
                            int mode = 0);

    int                 getIntSubelem (
			    char *name, 
			    int &valret, 
			    int dflt = 0);
    int                 setIntSubelem (char *name, int val);

    int                 getRealSubelem (
			    char *name, 
			    double &valret, 
			    double dflt = 0.);
    int                 setRealSubelem (char *name, double val);

    int                 getStringSubelem (
			    char *name, 
			    char **valret, 
			    char *dflt = 0);
    int                 setStringSubelem (char *name, char *val);

    int                 getRefSubelem (
			    char *name, 
			    DBelem_id &elemret, 
			    DBelem_id &dflt = DBnull_elem);
    int                 setRefSubelem (char *name, DBelem_id &val);

  private:
    DBelem_id           _id;
};

// In-line functions for DBelem_c

inline
DBelem_id DBelem_c::id () const
{
   return _id;
}

inline
Boolean  DBelem_c::operator == (DBelem_id rhs) const
{
    return (_id.proc_id == rhs.proc_id) && (_id.elem_id == rhs.elem_id);
}

inline
Boolean  DBelem_c::operator != (DBelem_id rhs) const
{
    return !((_id.proc_id == rhs.proc_id) && (_id.elem_id == rhs.elem_id));
}

inline
Boolean DBelem_c::isValid () const
{
    return DBvalid_elem (_id, DBnull_elem, 0);
}

inline
Boolean DBelem_c::isNil () const
{
    return DBis_null_elem (id ());
}

inline
Boolean DBelem_c::inSameProcess (DBelem_id elemId) const
{
    return _id.proc_id == elemId.proc_id;
}

inline
void DBelem_c::print (FILE *fp) const
{
    DBprint_elem_mode(_id, fp, 0);
}

inline
void DBelem_c::pushCtx ()
{
    DBpush_ctx (_id,DB_STATE_USR,0,0);
}

inline
void DBelem_c::popCtx ()
{
    DBpop_ctx (_id);
}


inline 
unsigned int DBelem_c::getSeq (int flags, DBelem_id tmplt) 
{ 
    return DBget_elem_seq (_id, tmplt, flags);
}

class DBfunc : public DBelem_c {
  public:
             DBfunc (DBelem_id id, char *name = 0) : DBelem_c (id, name) {}
    virtual ~DBfunc ();
    DBpfi  getFunc ();
    int    setFunc (DBpfi function);
};

#endif /* dbxx_h || DBelem_c */
/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/dbxx/int.h#1 $
*/
#ifndef int_h
#define	int_h		1

/////////////////////////////////////
class DBint: public DBelem_c {
/////////////////////////////////////
  public:
    // Constructor and Destructor
			DBint (
                            DBelem_id elemId, 
                            char *name = 0,
                            int mode = DB_ELEM_RD);
                        ~DBint ();

    // getting and setting
    int			getValue (int &value, int dflt = 0) const;
    int			setValue (int v);

    			operator int () const; 
    DBint & 		operator= (int rhs);
    int  		operator += (int rhs);
    int  		operator -= (int rhs);
    int  		operator == (int rhs) const;
    int  		operator != (int rhs) const;
    int  		operator < (int rhs) const;
    int  		operator <= (int rhs) const;
    int  		operator > (int rhs) const;
    int  		operator >= (int rhs) const;
};

inline
DBint::operator int () const { int value;  getValue (value); return value; }

inline
DBint & DBint::operator= (int rhs) { setValue (rhs); return *this; }

inline
int DBint::operator == (int rhs) const { return operator int () == rhs; }

inline
int DBint::operator != (int rhs) const { return operator int () != rhs; }

inline
int DBint::operator < (int rhs) const { return operator int () < rhs; }

inline
int DBint::operator <= (int rhs) const { return operator int () <= rhs; }

inline
int DBint::operator > (int rhs) const { return operator int () > rhs; }

inline
int DBint::operator >= (int rhs) const { return operator int () >= rhs; }

#endif /* int_h */
/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/dbxx/real.h#1 $
*/
#ifndef real_h
#define real_h	1

/////////////////////////////////////
class DBreal: public DBelem_c {
/////////////////////////////////////
  public:
    // Constructor and Destructor
			DBreal (
                            DBelem_id elemId, 
                            char *name = 0,
                            int mode = DB_ELEM_RD);
                        ~DBreal ();

    // getting and setting
    double		getValue (double &value, double dflt = 0.) const;
    int			setValue (double v);

    			operator double () const; 
    DBreal & 		operator= (double rhs);
    double		operator+= (double rhs);
    double		operator-= (double rhs);
    int			operator== (double rhs) const;
    int			operator!= (double rhs) const;
    int			operator < (double rhs) const;
    int			operator <= (double rhs) const;
    int			operator > (double rhs) const;
    int			operator >= (double rhs) const;
};

inline
DBreal::operator double () const { double v; getValue (v); return v; }

inline
DBreal & DBreal::operator= (double rhs) { setValue (rhs); return *this; }

inline int
DBreal::operator == (double rhs) const { return operator double () == rhs; }

inline int
DBreal::operator != (double rhs) const { return operator double () != rhs; }

inline int
DBreal::operator < (double rhs) const { return operator double () < rhs; }

inline int
DBreal::operator <= (double rhs) const { return operator double () <= rhs; }

inline int
DBreal::operator > (double rhs) const { return operator double () > rhs; }

inline int
DBreal::operator >= (double rhs) const { return operator double () >= rhs; }

#endif /* real_h */
/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/dbxx/ptr.h#1 $
*/
#ifndef ptr_h
#define ptr_h		1

/////////////////////////////////////
class DBpointer: public DBelem_c {
/////////////////////////////////////
  public:
    // Constructor and Destructor
			DBpointer (DBelem_id elemId, char *name = 0);
    virtual		~DBpointer ();

    // getting and setting

    DBelem_id &		getElemVal (DBelem_id &dflt = DBnull_elem);
    int			setElemVal (DBelem_id elem);
    virtual		operator DBelem_id () const;
    DBpointer &		operator= (DBelem_id elem_id);

    virtual Boolean	isNil () const;

    virtual Boolean	match (DBelem_id matchElem, int mode = DB_MATCH_POSS);
    virtual Boolean	match (char * tmplClass, int mode = DB_MATCH_POSS);

  private:
    DBelem_id		_value;
};

inline
DBpointer & DBpointer::operator= (DBelem_id elem_id) 
    { setElemVal (elem_id); return *this; }

inline
DBpointer::operator DBelem_id () const { return _value; }

#endif /* ptr_h */
/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/dbxx/dbstring.h#1 $
*/
#ifndef DBstring_h
#define DBstring_h	1

/////////////////////////////////////
class DBstring: public DBelem_c {
/////////////////////////////////////
  public:
    // Constructor and Destructor
			DBstring (
                           DBelem_id elemId, 
                           char *name = 0,
                           int mode = DB_ELEM_RD);
                        ~DBstring ();

    // getting and setting
    int                 setValue (char *str);
    int                 getValue (char **value, char *dflt = 0) const;
			operator char * () const;
    DBstring &		operator= (const char *rhs);
};

// In-line functions for DBstring

inline
DBstring::operator char * () const { char *v;  getValue (&v); return v; }

inline
DBstring & DBstring::operator= (const char *rhs)
{ 
    setValue ((char *) rhs);
    return *this;
}

#endif /* DBstring_h */
/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/dbxx/vctr.h#1 $
*/
#ifndef vctr_h
#define vctr_h	1

/////////////////////////////////////
class DBvector: public DBelem_c {
/////////////////////////////////////
  public:
    // Constructor and Destructor
                         DBvector (
			    DBelem_id elemId, 
			    char *name = 0, 
			    char *dimname = "dims");
                         ~DBvector () {};

    // getting and setting
    int                 size ();
    virtual void        reshape (int size);

    // Public Methods that need to be implemented for subclasses
    //  T operator [] (int i)
    //  T atGet (int i)
    //  void atSet (int i, T val)
  private:
    DBint                _dims;
};

// In-line functions for DBvector

#endif /* vctr_h */
/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/dbxx/sarray.h#1 $
*/
#ifndef sarray_h
#define sarray_h	1

/////////////////////////////////////
class DBstringArray: public DBvector {
/////////////////////////////////////
  public:
    // Constructor and Destructor
			DBstringArray (
			    DBelem_id elemId, 
			    char *name = 0, 
			    char *dimname = "dims");
    virtual		~DBstringArray ();

    // getting and setting
    void		atSet (int index, char *val);
    char *		atGet (int index);
    void                add (char *s);
    void                add (char **s, int count);
    char *		operator [] (int index);
};

// In-line functions for DBintVector

inline
DBstringArray::DBstringArray (DBelem_id elemId, char *name, char *dimname) :
    DBvector (elemId, name, dimname)
{
    // Empty
}

inline
char * DBstringArray::operator [] (int index)
{
    return atGet (index);
}

#endif /* sarray_h */
/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/dbxx/elemvctr.h#1 $
*/
#ifndef elemvctr_h
#define elemvctr_h	1

/////////////////////////////////////
class DBelemVector: public DBvector {
/////////////////////////////////////
  public:
    // Constructor and Destructor
			DBelemVector (
			    DBelem_id elemId, 
			    char *name = 0, 
			    char *dimname = "dims");
    virtual		~DBelemVector ();

    // getting and setting
    void		atSet (int index, DBelem_id idToAdd);
    DBelem_id		atGet (int index);
    DBelem_id		operator [] (int index);
    void		add (DBelem_id idToAdd);
    void		remove (DBelem_id idToRemove);
  private:
};

// In-line functions for DBelemVector

#endif /* elevctr_h */
/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/dbxx/intvctr.h#1 $
*/
#ifndef intvctr_h
#define intvctr_h	1

/////////////////////////////////////
class DBintVector: public DBvector {
/////////////////////////////////////
  public:
    // Constructor and Destructor
			DBintVector (
			    DBelem_id elemId, 
			    char *name = 0, 
			    char *dimname = "dims");
    virtual		~DBintVector ();

    // getting and setting
    void		atSet (int index, int val);
    int			atGet (int index);
    int			operator [] (int index);
  private:
    static int		_type;
};

// In-line functions for DBintVector

inline
DBintVector::DBintVector (DBelem_id elemId, char *name, char *dimname) :
    DBvector (elemId, name, dimname)
{
    // Empty
}

inline
int DBintVector::operator [] (int index)
{
    return atGet (index);
}

#endif /* intvctr_h */
